from ..widget_base import Input
from typing import Any


class CustomInput(Input):
    type = "custom-input"

    def __init__(self, key: str, html_body: str, **kwargs):
        super().__init__(key)
        self.set_props(dict(html_body=html_body, **kwargs))

    def set_props(self, props):
        self.html_body = props.get("html_body", "")
        self.label = props.get("label", "")
        self.html_head = props.get("html_head", "")
        self.css = props.get("css", "")
        self.js = props.get("js", "")
        self.value = props.get("initial_value", None)
        self.full_width = props.get("full_width", False)

    def serialize_value(self) -> Any:
        return self.value

    def render(self, context: dict):
        return {
            "type": self.type,
            "key": self.key,
            "label": self.label,
            "value": self.serialize_value(),
            "htmlBody": self.html_body,
            "htmlHead": self.html_head,
            "css": self.css,
            "js": self.js,
            "fullWidth": self.full_width,
        }

    def parse_value(self, value) -> Any:
        return value



import requests, inspect
from pathlib import Path
from concurrent import futures
from typing import Any, Callable, Tuple
from .credentials import get_credentials
from .utils import get_package_version, get_local_user_id

executor = futures.ThreadPoolExecutor()


def async_send_usage(data, header):
    api_url = "http://cloud-api.abstra.cloud/cli/editor/usage"
    requests.post(api_url, json=data, headers=header)


def usage(root_path: Path) -> Callable[..., Any]:
    def usage_decorator(func: Callable[..., Any]) -> Callable[..., Any]:
        def wrapper(*args: Tuple[Any], **kwargs: Any) -> Any:
            arg_names = inspect.getfullargspec(func).args
            arg_values = dict(zip(arg_names, args))

            metric_data = {
                "userId": get_local_user_id(),
                "payload": {**arg_values, **kwargs, **{"event": func.__name__}},
                "abstraVersion": get_package_version("abstra"),
            }

            headers = {"apiKey": get_credentials(root_path)}

            executor.submit(async_send_usage, metric_data, headers)

            return func(*args, **kwargs)

        original_route_name = func.__name__
        wrapper.__name__ = f"{original_route_name}_usage_wrapper"  ## avoid name collision in blueprints

        return wrapper

    return usage_decorator



import sys
from colorama import Fore, Style


def serve_message(port):
    print(
        Fore.MAGENTA
        + Style.BRIGHT
        + f"\n\nABSTRA EDITOR RUNNING: http://localhost:{port}/_editor\n\n"
        + Style.RESET_ALL
    )


def read_credentials():
    credentials = input(f"API Token: ")
    if not credentials:
        print("No API token configured")
        sys.exit(1)
    return credentials



import requests, os

DEFAULT_EXECUTE_URL = "https://cloud-api.abstra.cloud/cli/connectors/execute"
EXECUTE_URL = os.getenv("ABSTRA_CONNECTORS_EXECUTE_URL", DEFAULT_EXECUTE_URL)


class ConnectorExecutionError(Exception):  # public api
    def __init__(self, name: str, method_name: str, errors: list) -> None:
        self.name = name
        self.method_name = method_name
        self.errors = errors


class Method:
    def __init__(self, name: str, connector: "Connector") -> None:
        self.name = name
        self.connector = connector

    def __call__(self, **kwargs):  # public api
        j_body = {"name": self.connector.name, "method": self.name, "params": kwargs}

        headers = {
            "Content-Type": "application/json",
            "api-authorization": f"Bearer {os.getenv('ABSTRA_API_TOKEN')}",
        }

        if EXECUTE_URL == DEFAULT_EXECUTE_URL and not os.getenv("ABSTRA_API_TOKEN"):
            raise Exception("You must be logged in to execute a connector method")

        r = requests.post(EXECUTE_URL, headers=headers, json=j_body)
        if not r.ok:
            raise Exception(f"Error executing method {self.name}: {r.text}")

        result = r.json()
        if result["errors"]:
            raise ConnectorExecutionError(
                self.connector.name, self.name, result["errors"]
            )

        return result["returns"]


class Connector:
    def __init__(self, name: str) -> None:
        self.name = name

    def get_method(self, method_name: str) -> "Method":  # public api
        return Method(method_name, self)

    def execute(self, method_name: str, **kwargs):  # public api
        return self.get_method(method_name)(**kwargs)

    def __getattr__(self, method_name: str) -> "Method":
        return self.get_method(method_name)


def get_connector(name: str) -> "Connector":  # public api
    return Connector(name)



from .PandasRowSelectionInput import PandasRowSelectionInput
from .CodeInput import CodeInput
from .PlotlyOutput import PlotlyOutput
from .NumberInput import NumberInput
from .FileOutput import FileOutput
from .KanbanBoardInput import KanbanBoardInput
from .CurrencyInput import CurrencyInput
from .VideoInput import VideoInput
from .ToggleInput import ToggleInput
from .TextOutput import TextOutput
from .NpsInput import NpsInput
from .PasswordInput import PasswordInput
from .CheckboxInput import CheckboxInput
from .CardsInput import CardsInput
from .HtmlOutput import HtmlOutput
from .TimeInput import TimeInput
from .ProgressOutput import ProgressOutput
from .IframeOutput import IframeOutput
from .CpfInput import CpfInput
from .ImageOutput import ImageOutput
from .EmailInput import EmailInput
from .FileInput import FileInput
from .CnpjInput import CnpjInput
from .PandasOutput import PandasOutput
from .CustomInput import CustomInput
from .NumberSliderInput import NumberSliderInput
from .ListInput import ListInput
from .PhoneInput import PhoneInput
from .TextareaInput import TextareaInput
from .DropdownInput import DropdownInput
from .RichTextInput import RichTextInput
from .TextInput import TextInput
from .ImageInput import ImageInput
from .DateInput import DateInput
from .LinkOutput import LinkOutput
from .RatingInput import RatingInput
from .MultipleChoiceInput import MultipleChoiceInput
from .MarkdownOutput import MarkdownOutput
from .ClickInput import ClickInput
from .LatexOutput import LatexOutput
from .ChecklistInput import ChecklistInput
from .TagInput import TagInput



import requests, os, typing

DEFAULT_EXECUTE_URL = "https://cloud-api.abstra.cloud/cli/tables/execute"
EXECUTE_URL = os.getenv("ABSTRA_TABLES_EXECUTE_URL", DEFAULT_EXECUTE_URL)


def run(query: str, params: typing.Optional[typing.List] = None):  # public api
    j_body = {"query": query, "params": params or []}

    headers = {
        "Content-Type": "application/json",
        "api-authorization": f"Bearer {os.getenv('ABSTRA_API_TOKEN')}",
    }

    if EXECUTE_URL == DEFAULT_EXECUTE_URL and not os.getenv("ABSTRA_API_TOKEN"):
        raise Exception("You must be logged in to execute a tables query")

    r = requests.post(EXECUTE_URL, headers=headers, json=j_body)
    if not r.ok:
        raise Exception(f"Error executing query {query}: {r.text}")

    response = r.json()
    if response["errors"]:
        raise TablesExecutionError(response["errors"], query, params)

    return response["returns"]["result"]


class TablesExecutionError(Exception):  # public api
    def __init__(
        self,
        errors: typing.List,
        query: str,
        params: typing.Optional[typing.List] = None,
    ) -> None:
        self.query = query
        self.params = params
        self.errors = errors



from ..widget_base import Output
from typing import Any
import json


class PlotlyOutput(Output):
    type = "plotly-output"

    def __init__(self, fig: Any = None, **kwargs):
        self.set_props(dict(fig=fig, **kwargs))

    def set_props(self, props):
        self.fig = props.get("fig", None)
        self.full_width = props.get("full_width", False)
        self.label = props.get("label", None)

    def serialize_figure(self):
        if self.fig is None:
            import plotly.express as px

            df = px.data.tips()
            fig = px.density_heatmap(df, x="total_bill", y="tip")
            return json.loads(fig.to_json())
        return json.loads(self.fig.to_json())

    def render(self, context: dict):
        return {
            "type": self.type,
            "figure": self.serialize_figure(),
            "fullWidth": self.full_width,
            "label": self.label,
        }



_strings = {
    "en_us": {
        "error_required_field": "This field is required.",
        "error_invalid_email": "This email is invalid.",
        "error_invalid_cnpj": "This CNPJ is invalid.",
        "error_invalid_cpf": "This CPF is invalid.",
        "error_invalid_phone_number": "This phone number is invalid.",
        "error_min_list": "The minimum number of items is {min}.",
        "error_max_list": "The maximum number of items is {max}.",
        "error_min_number": "The minimum value is {min}.",
        "error_max_number": "The maximum value is {max}.",
    },
    "pt_br": {
        "error_required_field": "Este campo é obrigatório.",
        "error_invalid_email": "Este email é inválido.",
        "error_invalid_cnpj": "Este CNPJ é inválido.",
        "error_invalid_cpf": "Este CPF é inválido.",
        "error_invalid_phone_number": "Este número de telefone é inválido.",
        "error_min_list": "O número mínimo de itens é {min}.",
        "error_max_list": "O número máximo de itens é {max}.",
        "error_min_number": "O valor mínimo é {min}.",
        "error_max_number": "O valor máximo é {max}.",
    },
}


class Provider:
    def __init__(self, lang="en_us"):
        if lang not in _strings:
            raise Exception("Invalid language")

        self.lang = lang

    def get(self, key):
        return _strings[self.lang][key] if key in _strings[self.lang] else key



import sys, importlib
from pathlib import Path


def reload_modules_from_path(path: Path):
    for module in sys.modules.values():
        if hasattr(module, "__file__") and module.__file__:
            module_path = Path(module.__file__)
            if module_path.is_file() and module_path.parent == path:
                importlib.reload(module)



import traceback, flask_sock, typing, pathlib
from pathlib import Path
from ....session import LiveSession
from ...api.classes import FormJSON
from ....contract import forms_contract


def __wait_start(session: LiveSession):
    type = None
    while type != "start":
        type, data = session.recv()
    return data["params"]


def run_form(
    conn: flask_sock.Server,
    form_json: FormJSON,
    root_path: Path,
    session_id: typing.Optional[str] = None,
):
    session = LiveSession(conn, "forms", form_json.path)
    session.start_files_monitoring(root_path)
    if session_id is not None:
        session.id = session_id

    code_file_path = form_json.file
    code = pathlib.Path(code_file_path).read_text()

    namespace: dict = {}
    close_dto = forms_contract.CloseDTO(exit_code=0)
    __wait_start(session)
    session.send(forms_contract.SessionIdMessage(session.id))

    try:
        exec(code, namespace, namespace)
    except Exception as e:
        traceback.print_exc()
        close_dto.exception = e.__str__()
        close_dto.exit_code = 1
    finally:
        session.send(forms_contract.CloseMessage(close_dto))
        session.close()



import pkg_resources, os, requests
from ..utils.environment import is_preview


def check_latest_version(package_name="abstra"):
    if not is_preview():
        return

    try:
        current_version = pkg_resources.get_distribution(package_name).version
        response = requests.get(f"https://pypi.org/pypi/{package_name}/json")
        latest_version = response.json()["info"]["version"]

        if current_version == latest_version:
            print(f"Abstra Editor is up to date (version {current_version}).")
        else:
            print(
                f"A new version of Abstra Editor is available. Latest version is {latest_version}, but you have {current_version}."
            )
            print(f"Please run 'pip install {package_name} --upgrade' to update.")

    except pkg_resources.DistributionNotFound:
        print(f"{package_name} is not installed.")

    except requests.exceptions.RequestException:
        print(f"Unable to fetch version information for {package_name} from PyPI.")

    print("\n")
